package com.tzz.mvc.servlet;


import com.tzz.mvc.annaotation.*;
import com.tzz.mvc.controller.JvmController;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.*;

/**
 * @author : tangzhaozhen
 * @createTime : 2023/10/28 19:08
 */

public class DispatcherServlet extends HttpServlet {


    List<String> classNames = new ArrayList<String>();
    Map<String, Object>  beans = new HashMap<String,Object>();

    Map<String, Object> handerMap = new HashMap<String,Object>();


    public void init(ServletConfig config) throws ServletException  {
        //把所有的bean扫描---扫描所有的class文件
        scanPackage("com.tzz");
        //根据全类名创建bean
        doInstance();
        //根据bean进行依赖注入
        doIoc();
        //mvc/query简历映射管理 可以访问
        buildUrlMapping();
    }

    //com.tzz
    private void scanPackage(String basePackage) {
        URL url = this.getClass().getClassLoader().getResource("/"+basePackage.replaceAll("\\.", "/"));
        String fileStr = url.getFile();
        File file = new File(fileStr);
        //获取所有文件夹 或 文件
        String[] filesStr = file.list(); //mvc
        for (String path : filesStr) {
            File filePath = new File(fileStr+path); //com.tzz.mvc
            if (filePath.isDirectory()) {
                scanPackage(basePackage+"."+path);
            } else {
                //加入List
                classNames.add(basePackage+"."+filePath.getName());//com.tzz.mvc....class
            }
        }

    }

    private void doInstance() {
        if (classNames.size() <= 0) {
            System.out.println("包扫描失败-----");
            return;
        }
        //list的所有class类，对这些类进行切割
        for (String className : classNames) {
            String cn = className.replace(".class","");
            try {
                Class<?> clazz = Class.forName(cn);//com.tzz.mvc.com.tzz.mvc.aa.service.controller...
                if (clazz.isAnnotationPresent(EnjoyController.class)) {
                    Object instace = clazz.newInstance();   //创建控制类
                    EnjoyRequestMapping requestMapping = clazz.getAnnotation(EnjoyRequestMapping.class);
                    String rmvalue = requestMapping.value();
                    beans.put(rmvalue, instace);
                } else if (clazz.isAnnotationPresent(EnjoyService.class)) {
                    EnjoyService service = clazz.getAnnotation(EnjoyService.class);
                    Object instance = clazz.newInstance();
                    beans.put(service.value(), instance);
                } else {
                    continue;
                }


            } catch (ClassNotFoundException e) {
                throw new RuntimeException(e);
            } catch (InstantiationException e) {
                throw new RuntimeException(e);
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        }

    }

    //把service逻辑层 注入到 控制层controller
    public void doIoc() {
        if (beans.entrySet().size() <= 0) {
            System.out.println("没有一个实例化的类----");
        }
        //把map里面所有实例化的类遍历出来
        for (Map.Entry<String, Object> entry : beans.entrySet()) {
            Object instance = entry.getValue();
            Class<?> clazz = instance.getClass();
            if (clazz.isAnnotationPresent(EnjoyController.class)) {
                Field[] fields = clazz.getDeclaredFields();
                for (Field field : fields) {
                    if (field.isAnnotationPresent(EnjoyAutowired.class)) {
                        EnjoyAutowired auto = field.getAnnotation(EnjoyAutowired.class);
                        String key = auto.value(); //key JvmServiceIml
                        //开启权限 因为我们注入时用的private
                        field.setAccessible(true);
                        try {
                            field.set(instance, beans.get(key));
                        } catch (IllegalAccessException e) {
                            throw new RuntimeException(e);
                        }
                    } else {
                        continue;
                    }
                }
            } else {
                continue;
            }

        }


    }

    private void buildUrlMapping() {
        if (beans.entrySet().size() <= 0) {
            System.out.println("没有类的实例化----");
            return;
        }
        for (Map.Entry<String, Object> entry : beans.entrySet()) {
            Object instance = entry.getValue();
            Class<?> clazz = instance.getClass();

            if (clazz.isAnnotationPresent(EnjoyController.class)) {
                EnjoyRequestMapping requestMapping = clazz.getAnnotation(EnjoyRequestMapping.class);
                String classPath = requestMapping.value(); //mvc
                Method[] methods = clazz.getMethods();
                for (Method method : methods) {
                    if (method.isAnnotationPresent(EnjoyRequestMapping.class)) {
                        EnjoyRequestMapping methodAnnotation = method.getAnnotation(EnjoyRequestMapping.class);
                        String methodPath = methodAnnotation.value();
                        System.out.println("方法键为："+classPath + methodPath);
                        System.out.println("方法值为："+method   );
                        handerMap.put(classPath + methodPath, method);
                    } else {
                        continue;
                    }
                }
            }
        }

    }


    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        this.doPost(req, resp);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        // 获取请求路径
        String uri = req.getRequestURI();
        String context = req.getContextPath();
        String path = uri.replace(context, "");

        Method method = (Method) handerMap.get(path);

        JvmController instance = (JvmController) beans.get("/"+path.split("/")[1]);

        Object[] args = hand(req, resp, method);
        try {
            method.invoke(instance, args);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
    }



    private static Object[] hand(HttpServletRequest request, HttpServletResponse response, Method method) {
        //拿到待执行的方法有哪些参数
        Class<?>[] paramClazzs = method.getParameterTypes();

        //根据参数的个数，new 一个参数的数组，将方法的所有参数赋值到args
        Object[] args = new Object[paramClazzs.length];

        int args_i = 0;
        int index = 0;
        for (Class<?> paramClazz : paramClazzs) {
            if (ServletRequest.class.isAssignableFrom(paramClazz)) {
                args[args_i++] = request;
            }
            if (ServletResponse.class.isAssignableFrom(paramClazz)) {
                args[args_i++] = response;
            }
            Annotation[] paramAns = method.getParameterAnnotations()[index];
            if (paramAns.length > 0) {
                for (Annotation paramAn : paramAns) {
                    if (EnjoyRequestParam.class.isAssignableFrom(paramAn.getClass())) {
                        EnjoyRequestParam rp = (EnjoyRequestParam) paramAn;

                        args[args_i++] = request.getParameter(rp.value());
                    }
                }
            }
            index++;
        }
        return args;
    }


}
